### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces
from lovely.memcached.interfaces import IMemcachedClient
from zope.ramcache.interfaces.ram import IRAMCache

# import local interfaces
from ztfy.cache.interfaces import ICacheHandler

# import Zope3 packages
from zope.component import adapts
from zope.interface import implements

# import local packages


class RAMCacheAdapter(object):
    """RAM cache adapter"""

    adapts(IRAMCache)
    implements(ICacheHandler)

    def __init__(self, context):
        self.context = context

    def set(self, namespace, key, value):
        if not isinstance(key, dict):
            key = { '_key': key }
        self.context.set(value, namespace, key)

    def query(self, namespace, key, default=None):
        if not isinstance(key, dict):
            key = { '_key': key }
        return self.context.query(namespace, key, default)

    def invalidate(self, namespace, key=None):
        if (key is not None) and not isinstance(key, dict):
            key = { '_key': key }
        self.context.invalidate(namespace, key)

    def invalidateAll(self):
        self.context.invalidateAll()


class MemcachedCacheAdapter(object):
    """Memcached cache adapter"""

    adapts(IMemcachedClient)
    implements(ICacheHandler)

    def __init__(self, context):
        self.context = context

    def set(self, namespace, key, value):
        self.context.set(value, key, ns=namespace)

    def query(self, namespace, key, default=None):
        return self.context.query(key, default, ns=namespace)

    def invalidate(self, namespace, key=None):
        self.context.invalidate(key, ns=namespace)

    def invalidateAll(self):
        self.context.invalidateAll()



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.cache.metadirectives import ICacheProxyHandlerBase

# import Zope3 packages
from zope.component.security import PublicPermission
from zope.component.zcml import utility

# import local packages
from ztfy.cache.utility import CacheProxyHandler


def cache_handler(context, name='', cache_interface=None, cache_name=None):
    """Register cache proxy from ZCML"""
    handler = CacheProxyHandler(name, cache_interface, cache_name)
    utility(context, ICacheProxyHandlerBase, component=handler, permission=PublicPermission, name=name)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.cache.metadirectives import ICacheProxyHandlerBase

# import Zope3 packages
from zope.interface import Interface
from zope.schema import TextLine, Choice

# import local packages

from ztfy.cache import _


class ICacheHandler(Interface):
    """ICacheHandler is a common basic interface to several cache managers"""

    def set(self, namespace, key, value):
        """Set the given value for the given key"""

    def query(self, namespace, key, default=None):
        """Query cache for given key"""

    def invalidate(self, namespace, key):
        """Invalidate key in given namespace"""

    def invalidateAll(self):
        """Invalidate all cache entries"""


class IPersistentCacheProxyHandlerInfo(Interface):
    """Cache proxy handler interface"""

    cache_interface = Choice(title=_("Cache utility interface"),
                             required=True,
                             vocabulary="Interfaces")

    cache_name = TextLine(title=_("Cache utility name"),
                          required=False)


class IPersistentCacheProxyHandler(ICacheProxyHandlerBase, IPersistentCacheProxyHandlerInfo):
    """Persistent cache proxy handler interface"""



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages
from persistent import Persistent

# import Zope3 interfaces

# import local interfaces
from ztfy.cache.interfaces import ICacheHandler, IPersistentCacheProxyHandler
from ztfy.cache.metadirectives import ICacheProxyHandler

# import Zope3 packages
from zope.component import queryUtility
from zope.container.contained import Contained
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty

# import local packages


class CacheProxyHandler(object):
    """Cache proxy handler"""

    implements(ICacheProxyHandler)

    name = FieldProperty(ICacheProxyHandler['name'])
    cache_interface = FieldProperty(ICacheProxyHandler['cache_interface'])
    cache_name = FieldProperty(ICacheProxyHandler['cache_name'])

    def __init__(self, name, cache_interface, cache_name):
        self.name = name
        self.cache_interface = cache_interface
        self.cache_name = cache_name

    def getCache(self):
        return ICacheHandler(queryUtility(self.cache_interface, self.cache_name), None)


class PersistentCacheProxyHandler(Persistent, Contained):
    """Persistent cache proxy handler"""

    implements(IPersistentCacheProxyHandler)

    cache_interface = FieldProperty(IPersistentCacheProxyHandler['cache_interface'])
    cache_name = FieldProperty(IPersistentCacheProxyHandler['cache_name'])

    def getCache(self):
        return ICacheHandler(queryUtility(self.cache_interface, self.cache_name), None)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
from zope.configuration.fields import GlobalInterface
from zope.interface import Interface
from zope.schema import TextLine

# import local packages

from ztfy.cache import _


class ICacheProxyHandlerBase(Interface):
    """Cache proxy handler base interface"""

    def getCache(self):
        """Get the real cache utility"""


class ICacheProxyHandlerInfo(Interface):
    """A cache handler is a proxy to an actual cache utility"""

    name = TextLine(title=_("Cache handler name"),
                    required=False)

    cache_interface = GlobalInterface(title=_("Cache utility interface"),
                                      required=True)

    cache_name = TextLine(title=_("Cache utility name"),
                          required=False)


class ICacheProxyHandler(ICacheProxyHandlerBase, ICacheProxyHandlerInfo):
    """Cache proxy handler interface"""
